unit unodos;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  xmatdefs, usimplex, uglobs,
  SysUtils,
  Classes,
  ucosa,
  uCosaConNombre,
  uActores,
  //uINodos,
  ufichasLPD,
  ufechas;

resourcestring
  rsNodo = 'Nodo';

type

  { TListNodo }

  TListNodo=class(TListaDeCosasConNombre)
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
  end;

  { TNodo }

  TNodo = class(TActor)
    // resultados
    cmarg: TDAOfNReal;
    ZonaFlucar: integer;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades: TFichasLPD; ZonaFlucar: integer);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PrepararMemoria(globs: TGlobs); override;

    function InfoAd_: string; override;
    class function DescClase: string; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;
    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;
{$IFDEF SPXCONLOG}
    procedure spx_NombrarVariables(s: TSimplex); override;
{$ENDIF}
    function getNombreVar(ivar: integer; var nombre: string): boolean; override;
    function getNombreRes(ires: integer; var nombre: string): boolean; override;

    procedure Free; override;

    procedure PubliVars; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;

  end;



procedure AlInicio;
procedure AlFinal;

implementation

{ TListNodo }

class function TListNodo.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: Integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TListNodo, TNodo, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result:=list;
end;

//-------------------
// Métodos de TNodo
//===================
constructor TNodo.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades: TFichasLPD; ZonaFlucar: integer);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  Self.ZonaFlucar := ZonaFlucar;
end;

constructor TNodo.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  ZonaFlucar := 0;
  if f.Version >= 66 then
  begin
    f.rd('ZonaFlucar', ZonaFlucar);
  end;
end;

procedure TNodo.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('ZonaFlucar', ZonaFlucar);
end;

procedure TNodo.PrepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
  setlength(cmarg, globs.NPostes);
end;

function TNodo.InfoAd_: string;
begin
  Result := '';
end;

class function TNodo.DescClase: string;
begin
  Result := rsNodo;
end;

procedure TNodo.opt_nvers(var ivar, ivae, ires: integer);
begin
  // el nodo no agreaga variables de control
  Self.ires := ires;
  // El NODO agrega una restricción por cada POSTE
  ires := ires + globs.NPostes;
end;

procedure TNodo.opt_cargue(s: TSimplex);
{$IFDEF RNODO_IGUALDAD}
var
  ip: integer;
{$ENDIF}
begin
  // nada, cada demanda carga el termino constante y cada generador los variables
{$IFDEF SPXCONLOG}
  spx_NombrarVariables(s);
{$ENDIF}
{$IFDEF RNODO_IGUALDAD}
  for ip := 0 to globs.NPostes - 1 do
    s.FijarRestriccionIgualdad(ires + ip);
{$ENDIF}
end;

procedure TNodo.opt_fijarRestriccionesDeCaja(s: TSimplex);
begin
  // no tiene variables no tiene restricciones de caja
end;

procedure TNodo.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;
begin
  // no tiene variables no tiene que leer las variables.
  // lo que podemos leer es el multiplicador de Lagrange para tener
  // el costo marginal del nodo.
  for iposte := 0 to globs.NPostes - 1 do
    cmarg[iposte] := -s.ymult(ires + iposte) * globs.invDurPos[iposte];
end;

{$IFDEF SPXCONLOG}
procedure TNodo.spx_NombrarVariables(s: TSimplex);
var
  iposte: integer;
begin
  for iposte := 0 to globs.NPostes - 1 do
    s.set_NombreRest(ires + iposte, nombre + 'res-nodo' + IntToStr(iposte + 1));
end;

{$ENDIF}

function TNodo.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  Result := False;
end;

function TNodo.getNombreRes(ires: integer; var nombre: string): boolean;
begin
  if (ires >= self.ires) and (ires < self.ires + globs.NPostes) then
  begin
    nombre := self.nombre + 'res-nodo' + IntToStr(ires - self.ires + 1);
    Result := True;
  end
  else
    Result := False;
end;

procedure TNodo.Free;
begin
  setlength(cmarg, 0);
  inherited Free;
end;

procedure TNodo.PubliVars;
begin
  inherited PubliVars;

  PublicarVariableVR('cmg', '[USD/MWh]', 6, 1, cmarg, True, True);
end;


class function TNodo.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: Integer): TDataColumnListOfCosa;
var
 list: TDataColumnListOfCosa;
begin
  //list:= TDataColumnListOfCosa.Create(TNodo, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('ZonaFlucar', TNodo(nil).ZonaFlucar, TDataTypeIntegerOfCosa, '', 113, -1);
  Result:=list;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TNodo.ClassName, TNodo);
  registrarClaseDeCosa(TListNodo.ClassName, TListNodo);
end;

procedure AlFinal;
begin
end;

end.
