{$DEFINE NO_USAR_RND_COMPILADOR}
unit fddp;

{+doc
+NOMBRE: fddp
+CREACION: 18.3.1992 ..
+AUTOR: Ruben Chaer.
+REVISION:
+AUTOR:
+REGISTRO:
+TIPO: Unidad Pascal.
+PROPOSITO: Funciones de densidad de probabilidad.
+PROYECTO: rchlib
+DESCRIPCION:
  Implementacin de un conjunto de funciones tiles para estadsticas.
  2.9.2005 le agrego   Tf_ddp_VectDeMuestras = class( Tf_ddp )
  y paso a class (salve el cdigo viego como old_fddp.pas.
-doc}
interface

uses
  SysUtils,
  xMatDefs,
  Math,
  MatReal,
  math01,
  math02,
  udisnormcan;

type
  {$IFDEF NO_USAR_RND_COMPILADOR}
  {$IFDEF RAND3_REAL}// si la mquine tiene enteros de menos de 4 bytes
  T_glma_data= array [1..55] of real;
  {$ELSE}
  T_glma_data= array [1..55] of integer;
  {$ENDIF}
  {$ENDIF}


  TMadreUniforme = class; // la defino ms adelante

  // Almacena el estado de la fuente aleatoria.
  // Como las fuentes son generadas a partir de una MADRE_UNIFORME
  // el estado es el de la madre uniforme.
  TEstadoMadreUniforme = record
    semilla_base: integer;
{$IFDEF NO_USAR_RND_COMPILADOR}
    glma: T_glma_data;
    glinext, glinextp: integer;
{$ELSE}
    semilla: integer;
{$ENDIF}
  end;
  PEstadoMadreUniforme = ^TEstadoMadreUniforme;


  Tf_ddp = class
  private
    semilla_base: integer;
    // semilla Es solo para inicializar. Luego no representa el estado
    flg_BorrarMadreUnif: boolean;
  public
    madreUnif: TMadreUniforme;
    dtmax: NReal;
    area_t0, t0, area_t1, t1: NReal;
    area_u_t, u_t: NReal;
    g0: NReal;

    //Este constructor solo lo debe llamar TMadreUniforme o una
    //fuente que no precise de un generador de sorteos uniformes como
    //Tf_ddp_VectDeMuestras
    constructor Create(xt0, xarea_t0, xt1, xarea_t1: NReal; semilla: integer); overload;

    // Atencin, madreUniforme <> nil implica que se usar la semilla global
    // se ignora el valor de la semilla.
    // si madreUniforme = nil implica esta fuente usa su propia semilla y por lo tanto
    // es independiente de las demas y se crea una madreUniforme con la semilla pasada
    constructor Create(xt0, xarea_t0, xt1, xarea_t1: NReal;
      madreUniforme: TMadreUniforme; semilla: integer); overload;

    procedure Free; virtual;
    function densidad(x: NReal): NReal; virtual;
    function distribucion(x: NReal): NReal; virtual;
    function area_t(t: NReal): NReal; virtual;
    function area_t_rapida(t: NReal): NReal; virtual;
    function t_area(area: NReal): NReal; virtual;
    function rnd: NReal; virtual;

    // reinicia el generador de nmeros aleatorios.
    procedure Reiniciar(nueva_semilla_base: integer); virtual;
    function get_semilla_base: integer;

  end;


(*+doc TMadreUniforme
Esta fuente es especial. Las fuentes de este tipo son independientes en el uso
de las semillas.
Hay una instancia de este tipo global de la unidad que es la que usan las dems
fuentes cuando especifican semilla=0 al inicializarse.
-doc*)
  TMadreUniforme = class(Tf_ddp)
  public
    {$IFDEF NO_USAR_RND_COMPILADOR}
    glinext, glinextp: integer;
    glma: T_glma_data;
    {$ELSE}
    semilla: integer;
    {$ENDIF}
    constructor Create(semilla: integer);
    procedure Free; override;
    function densidad(x: NReal): NReal; override;
    function distribucion(x: NReal): NReal; override;
    function area_t(t: NReal): NReal; override;
    function area_t_rapida(t: NReal): NReal; override;
    function t_area(area: NReal): NReal; override;
    function rnd: NReal; override;

    //Retorna un entero en todo el rango posible [-MaxInt-1, MaxInt]
    function randomInt: integer;

    //Los extremos del rango son inclusive
    function randomIntRange(const minRango, maxRango: integer): integer;

    //fija la semilla aleatoria para la fuente, dos fuentes inicializadas con
    //la misma semilla generaran la misma secuencia de nmeros, pero una fuente
    //inicializada con la misma semilla que otra puede no generar los mismos
    //valores pues su glinext y glinextp se inicializan en valores arbirtrarios
    //no necesariamente iguales a los de la fuente original
    procedure Reiniciar(semilla: integer); override;

    //Fija el estado de la fuente, si a una fuente se le fija el estado de otra
    //generaran la misma secuencia de numeros
    procedure setEstado(const estado: TEstadoMadreUniforme); overload;
    procedure setEstado(const otraFuente: TMadreUniforme); overload;
    procedure getEstado(var estado: TEstadoMadreUniforme);
  end;

  Pf_ddpMadreUniformeRand3 = ^TMadreUniforme;



(*+doc Fuente uniforme en el rango 0-1 puede
  Es para usar en los lugares dnde se quiera tener una uniforme en el que
  el manejo de las semillas pueda ser el global.
-doc*)
  Tf_ddpUniformeRand3 = class(Tf_ddp)
  public
    constructor Create(madreUniforme: TMadreUniforme; semilla: integer);
    function densidad(x: NReal): NReal; override;
    function distribucion(x: NReal): NReal; override;
    function area_t(t: NReal): NReal; override;
    function area_t_rapida(t: NReal): NReal; override;
    function t_area(area: NReal): NReal; override;
    function rnd: NReal; override;
  end;

  Tf_ddp_GaussianaNormal = class(Tf_ddp)
  public
    gliset: integer;
    glgset: NReal;
    constructor Create(madreUniforme: TMadreUniforme; semilla: integer);
    function densidad(x: NReal): NReal; override;
    function distribucion(x: NReal): NReal; override;
    function area_t(t: NReal): NReal; override;
    function area_t_rapida(t: NReal): NReal; override;
    function t_area(area: NReal): NReal; override;
    function rnd: NReal; override;
    procedure Reiniciar(semilla: integer); override;
  end;

  Tf_ddp_Student = class(Tf_ddp)
  public
    nu, y0: NReal;
    constructor Create(N: integer; madreUniforme: TMadreUniforme; semilla: integer);
    function densidad(x: NReal): NReal; override;
  end;

  Tf_ddp_ChiCuadrado = class(Tf_ddp)
  public
    nu, y0: NReal;
    constructor Create(N: integer; madreUniforme: TMadreUniforme; semilla: integer);
    function densidad(x: NReal): NReal; override;
  end;

  Tf_ddp_Weibull = class(Tf_ddp)
  public
    k, lambda: NReal;
    constructor Create(ValorMedio, Constante_k: NReal; madreUniforme: TMadreUniforme;
      semilla: integer);


    // estima a partir del vector de muestras los valores de
    // k y lambda que mejor ajustan
    // OJO!, se supone que los datos tienen que venir ordenados en forma creciente
    // si no lo estn usar vdatos.sort(true) antes de llamar este constructor.
    constructor Create_vmuestras(
      vdatos: TVectR;
      madreUniforme: TMadreUniforme; semilla: integer);


    procedure setNuevosParams(ValorMedio, Constante_k: NReal);
    function densidad(x: NReal): NReal; override;
    function distribucion(x: NReal): NReal; override;
    function area_t(t: NReal): NReal; override;
    function area_t_rapida(t: NReal): NReal; override;
    function t_area(area: NReal): NReal; override;

  private
    kSobreLambda: NReal;
    unoSobreK: NReal;
  end;

  Tf_ddp_VectDeMuestras = class(Tf_ddp)
  public
    a: TVectR;

    constructor Create;  // usar uno de los dos siguientes

    // esta versin hace una copia del vector muestras.
    constructor Create_ClonandoMuestras(muestras: TVectR;
      madreUniforme: TMadreUniforme;   semilla: integer);

    // esta versin no copia el vector de muestras sino que lo apunta OJO.
    // cualquier modificacin externa modifica la fuente.
    constructor Create_SinClonarMuestras(muestras: TVectR; madreUniforme: TMadreUniforme; semilla: integer);

    procedure Free; override;

(* retorna el k tal que a.e(k) <= t <= a.e(k+1)
  si retorna 0 significa que t < a.e(1)
*)
    function k_inf_t(t: NReal): integer;
    function densidad(x: NReal): NReal; override;
    function area_t(t: NReal): NReal; override;
    function t_area(area: NReal): NReal; override;
  end;

  Tf_ddp_VectFPAcum = class(Tf_ddp)
  public
    a: TVectR;
    dt: NReal;
    constructor Create(
    // el primer casillero es el de xt0, y el ltimo elde xt1
    // el primer casillero debiera tener valor 0 y el ltimo 1 para
    // que xt1, xt2 comprenda al rango posible de la variable
      FAcum: TVectR; xt0, xt1: NReal; madreUniforme: TMadreUniforme;
      semilla: integer);
    procedure Free; override;

(* retorna el k tal que a.e(k) <= u <= a.e(k+1)
  si retorna 0 significa que u < a.e(1)

  u debe estar entre [0, 1]
*)
    function k_inf_u(u: NReal): integer;
    function densidad(x: NReal): NReal; override;
    function area_t(t: NReal): NReal; override;
    function t_area(area: NReal): NReal; override;
  end;

(*
  Donde pol(t) es un polinimo de t
   La funcin de probabilidad acumulada del tipo F(t) = 1 - exp( pol(t) )
   La funcin es vlida en el rango xt0, xt1
*)
  Tf_ddp_F1mExpPol = class(Tf_ddp)
  public
    pol, mdpol: TVectR;
    constructor Create(xpol: TVectR; xt0, xt1: NReal;
      madreUniforme: TMadreUniforme; semilla: integer);
    procedure Free; override;

    function densidad(x: NReal): NReal; override;
    function area_t(t: NReal): NReal; override;
  end;



(*+doc CambiarHistograma
 rch@jueves 3 de setiembre del 2009.

 cambiarHistograma, dado un vector de muestras x con su vector de pesos q
 busca el vector de pesos p tal que la serie x con los pesos p tenga
 el valor esperado my y la desviacin estndar stdy.

 El resultado dela funcin es
   0 si logr el objetivo.
  -1 la varianza no es alcanzable. Ver ratio para informacin de la mxima varianza alcanzable
  -2 el valor my es superior a x(N)
  -3 el valor my es inferior a x(1)
  -4 super las N itereaciones en bsqueda del objetivo.

 El resultado -4 se da cuando la varianza exigida es muy pequea y no logra
 conseguir un su
 conjunto de muestras que combinadas del el valor medio objetivo
 con la varianza requeirda

 Si logra el objetivo crea y retorna el vector de pesos p, si no
 logra el objetivo p se retorna a nil.
 Tambin devuelve en cnt_Iteraciones la cantidad de iteraciones que tuvo
 que hacer para resolver el problema.

 En la variable "ratio" se devuelve la relacin entre la desviacin estandar
 objetivo y la desviacin estndar  mxima alcanzable con el rango del vector
 de muestras para el valor esperado objetivo.
 Si ratio > 1 no es posible encontra solucin.
 Para valores de 0<= ratio < 1 el algoritmo debiera encontrar la solucin.

 Para la bsqueda de la solucin se resuelve el siguiente problema de optimizacin.

 min  ( sum( ( p[i] - q[i] )^2 ) ;
 p[i]

 sujeto a:
   sum( p[i] ) = 1 ;
   < p[i] * x[i] > = my
   < p[i] * ( x[i] - my )^2 > = stdy^2
   p[i] >= 0 ;

 El resultado es por tanto un vector de pesos lo ms cercano posible al original
 que cumpla que con ese vector de pesos las muestras x[i] tienen valor esperado
 y desviacin estndar dados.
-doc*)

function cambiarHistograma(const x: TVectR; // vector de muestras
  const q: TVectR; // vector de pesos de las muestras NIL = equiprobables.

  my: NReal; // valor esperado objetivo
  stdy: NReal; // valor de la desviacin estandar objetivo

  var p: TVectR; // vector con los nuevos pesos
  var ratio: NReal; // relacin entre la stdy mxima alcanzable y la objetivo
  var cnt_Iteraciones: integer // cantidad de iteraciones
  ): integer;
// retorna 0 si logr el objetivo.
// -1 la varianza no es alcanzable. Ver ratio para informacin de la mxima varianza alcanzable
// -2 el valor my es superior a x(N)
// -3 el valor my es inferior a x(1)
// -4 sal por numero maximo de iteraciones.


(*+doc muestrasEquiprobables

Recibe un histograma descripto como un vector de muestras (x) y un vector de pesos (p)
y retorna un vector N, muestras equi-probables que representa el mismo histograma.

-doc*)
function muestrasEquiprobables(const x: TVectR;
  const p: TVectR;
  n: integer): TVectR;


(* factorial *)
function factorial( n: integer ): integer;

(* CombinacionesNP - Cominaciones de N tomadas de a P *)
function combinaciones_mn( m, n : integer ): integer;


(*
Retorna un vector con la CDF de la distribucin Binomial de
m elementos con probabilidad p.

La variable aletaoria es la cantidad de elementos en estado 1
en una poblacin de m elmentos de los que cada uno en forma
independiente puede estar en estado 1 con probablilidad p y en
estado 0 con probabilidad 1-p

El resultado tiene m+1 elementos.
v[k] es la probabilidad acumulada de tener j < k elementos en estado 1.

As, el primer casillero v[1] tiene la probabilidad de tener 0 elementos
en estado 1. El primier elemento tiene el valor (1-p)^m

El casillero v[m+1] tiene la probabilidad de tener m o menos elementos en
estado 1. Como siempre se tendr del conjunto de m elemetos j <=m <(m+1)
elementos en estado 1, v[m+1] = 1.
*)
function binomial_cdf( m: integer; p: NReal ): TVectR;

(*
Retorna un vector con la PDF de la funcin Densidad de Probabilidad Binomial
de m elementos con probabilidad p.

La variable aletaoria es la cantidad de elementos en estado 1
en una poblacin de m elmentos de los que cada uno en forma
independiente puede estar en estado 1 con probablilidad p y en
estado 0 con probabilidad 1-p

El resultado tiene m+1 elementos.
v[k] es la probabilidad acumulada de tener  k-1 elementos en estado 1.

As, el primer casillero v[1] tiene la probabilidad de tener 0 elementos
en estado 1. El primier elemento tiene el valor (1-p)^m

El casillero v[m+1] tiene la probabilidad de tener m elementos en
estado 1.  v[m+1] = p^m.

El casillero v[k+1] tienen la probabilidad de tener k elementos
en estado 1 y m-k elementos en estado 0.
  v[k+1] = C(m,n) p^k (1-p)^(m-k)

*)
function binomial_pdf( m: integer; p: NReal ): TVectR;


implementation

(* factorial *)
function factorial( n: integer ): integer;
var
  res: integer;
  k: integer;
begin
  if n > 0 then
  begin
    res:= 1;
    for k:= 2 to n do
      res:= res * k;
    result:= res;
  end
  else
   if n = 0 then
     result:= 1
   else
   begin
     res:= - n;
     if (res mod 2) = 0 then
        result:= factorial( res )
     else
        result:= - factorial( res );
   end;
end;

(* Combinaciones_mn - Cominaciones de m tomadas de a n *)
function combinaciones_mn( m, n : integer ): integer;
var
  r: NReal;
  k, j: integer;
begin
  if (m-n) < n then // busco calcular con el n ms chico para reducir el for
    result:= combinaciones_mn( m, m-n )
  else
  begin
    r:= 1.0;
    for k:= 1 to n do
      r:= r * (m-n+k)/k;
    result:= trunc( r+0.5 );
  end;
end;

(*
Usamos que C(m, n ) = C(m, n -1) * (m-(n-1))/n
para calcular la distribucin Binomial
*)
function binomial_pdf( m: integer; p: NReal ): TVectR;
var
  c: NReal;
  k: integer;
  res: TVectR;
  q, psq: NReal;
begin
  q:= 1-p;
  c:= 1;
  res:= TVectR.Create_Init( m + 1);
  psq:= p/q;
  c:= power( q, m );
  for k:= 1 to m+1 do
  begin
   res.pv[k]:= c;
   c:= c * (m - (k-1) )/ k*psq;
  end;
  result:= res;
end;


(*
Ca y la acumulamos
para tener la CDF
*)
function binomial_cdf( m: integer; p: NReal ): TVectR;
var
  k: integer;
  cdf: TVectR;
begin
  cdf:= binomial_pdf( m, p );
  for k:= 2 to cdf.n do
   cdf.pv[k]:= cdf.pv[k-1] + cdf.pv[k];
  result:= cdf;
end;


function cambiarHistograma(const x: TVectR; // vector de muestras
  const q: TVectR; // vector de pesos de las muestras NIL = equiprobables.

  my: NReal; // valor esperado objetivo
  stdy: NReal; // valor de la varianza objetivo

  var p: TVectR; // verctor con los nuevos pesos
  var ratio: NReal; // relacin entre la stdy mxima alcanzable y la objetivo
  var cnt_Iteraciones: integer // cantidad de iteraciones
  ): integer;

label
  lbl_Iterar, lbl_finiter;

var
  k: integer;
  N: integer;
  mx, varx: NReal;
  vary: NReal;
  m, landa: TMatR;
  a: NReal;

  activa: array of boolean;
  beta: TVectR;
  {  cntActivas, }cntVioladas: integer;
  //  j: integer;

  amp, mpu, varpu: NReal;
  invertible: boolean;
  c: array[1..3] of NReal;
  jFil, jCol: integer;

  qaux: TVectR;
  buscando: boolean;
  flg_SupereNiteraciones: boolean;
  e10: integer;

begin
  p := nil;
  flg_SupereNIteraciones := False;

  N := x.n;

  if (my < x.e(1)) then
  begin
    Result := -3;
    exit;
  end;

  if (my > x.e(N)) then
  begin
    Result := -2;
    exit;
  end;


  vary := sqr(stdy);
  if q = nil then
  begin
    qaux := TVectR.Create_Init(N);
    a := 1.0 / N;
    for k := 1 to N do
      qaux.pon_e(k, a);
  end
  else
    qaux := q;

  x.promedioVarianza(mx, varx);
{  mx:= x.promedio;
  varx:= x.varianza;}

  amp := x.e(N) - x.e(1);
  mpu := (my - x.e(1)) / amp;
  varpu := vary / sqr(amp);

  ratio := sqrt(varpu / (mpu * (1 - mpu)));

  if ratio > 1 then
  begin
    Result := -1;
    exit;
  end;

  p := TVectR.Create_init(x.n);
  beta := TVectR.Create_init(x.n);
  setlength(activa, N + 1);
  landa := TMatR.Create_Init(3, 1);
  m := TMatR.Create_Init(3, 3);

  for k := 1 to N do
    activa[k] := False;
  //  cntVioladas:= 0;
  //  cntActivas:= 0;
  cnt_Iteraciones := 0;
  buscando := True;

  lbl_iterar:
    Inc(cnt_Iteraciones);

  if (cnt_Iteraciones > N) then
  begin
    //    da warning, en el if despues de lbl_finiter se asigna el valor de result en todas las ramas
    //    result:= -4;
    buscando := False;
    flg_SupereNIteraciones := True;
    goto lbl_finiter;
  end;

  // cargo el trmino independiente
  landa.pon_e(1, 1, 0);
  landa.pon_e(2, 1, 2 * mx - 2 * my);
  a := 2 * (varx + sqr(mx - my)) - 2 * vary;
  landa.pon_e(3, 1, a);

  m.Ceros;

  // cargo la matriz
  for k := 1 to N do
  begin
    c[1] := 1;
    c[2] := x.e(k);
    c[3] := sqr(x.e(k) - my);

    if not activa[k] then
    begin
      for jFil := 1 to 3 do
        for jCol := jFil to 3 do
          m.acum_e(jFil, jCol, c[jFil] * c[jCol]);
    end
    else
    begin
      landa.acum_e(1, 1, -2 * qaux.e(k) * c[1]);
      landa.acum_e(2, 1, -2 * qaux.e(k) * c[2]);
      landa.acum_e(3, 1, -2 * qaux.e(k) * c[3]);
    end;
  end;

  // simetrizo la matriz
  for jFil := 2 to 3 do
    for jCol := 1 to jFil - 1 do
      m.pon_e(jFil, jCol, m.e(jCol, jFil));


  // resolvemos el problema
  {  a:=  }m.Escaler(landa, invertible, e10);
  if not invertible then
    goto lbl_finiter;

  // calculamos los betas ; si alguno da menor que cero tengo que
  // desactivar la restriccin. pi >= 0 correspondiente.
  for k := 1 to N do
  begin
    if activa[k] then
    begin
      a := landa.e(1, 1) + x.e(k) * landa.e(2, 1) + sqr(x.e(k) - my) *
        landa.e(3, 1) - 2.0 * qaux.e(k);
      beta.pon_e(k, a);
      if a < -0.000000000001 then
      begin
        activa[k] := False;
        //        dec( cntActivas );
        goto lbl_iterar;
      end;
    end;
  end;


  // bien, si todos los betas son positivos calculo pos pi y veo si tengo nuevas
  // violaciones.


  cntVioladas := 0;
  for k := 1 to N do
  begin
    if not activa[k] then
    begin
      a := qaux.e(k) - 0.5 * (landa.e(1, 1) + landa.e(2, 1) * x.e(k) +
        landa.e(3, 1) * sqr(x.e(k) - my));
      p.pon_e(k, a);
      if a < -0.00000000001 then
      begin
        Inc(cntVioladas);
        activa[k] := True;
      end;
    end
    else
      p.pon_e(k, 0);
  end;

  if (cntVioladas > 0) then
  begin
    //    cntActivas:= cntActivas + cntVioladas;
    goto lbl_iterar;
  end;

  buscando := False;

  lbl_finiter:

    if q = nil then
      qaux.Free;

  beta.Free;
  setlength(activa, 0);
  landa.Free;
  m.Free;

  if buscando then
  begin
    p.Free;
    p := nil;
    Result := -1;
  end
  else if flg_SupereNIteraciones then
  begin
    Result := -4;
  end
  else
    Result := 0;
end;



function muestrasEquiprobables(const x: TVectR;
  const p: TVectR;
  n: integer): TVectR;
var
  FdpAcum: TVectR;
  k: integer;
  Np: integer;
  res: TVectR;
  //  buscando: boolean;
  ir: NReal;
  x0, x1: NReal;
  jBuscando: integer;
  pAcum_j, pAcum_k: NReal;
  pAcum_jm1, delta_pAcum_j: NReal;
  y: NReal;
begin
  Np := p.n;
  FdpAcum := TVectR.Create_Init(Np);
  FdpAcum.pon_e(1, p.e(1));
  for k := 2 to Np do
    FdpAcum.pon_e(k, FdpAcum.e(k - 1) + p.e(k));

  res := TVectR.Create_Init(N);
  jBuscando := 1;
  pAcum_j := fdpAcum.e(1);
  delta_pAcum_j := 0; // lo asigno para eliminar warning.
  pAcum_jm1 := 0; // lo asigno para eliminar warning.
  x0 := x.e(1);
  x1 := x0;

  k := 1;
  pAcum_k := k * 1.0 / N;

  while k <= N do
  begin
    if pAcum_k <= pAcum_j then
    begin
      if (jBuscando > 1) and (delta_pAcum_j > AsumaCero) then
      begin
        ir := (pAcum_k - pAcum_jm1) / delta_pAcum_j;
        y := x0 + (x1 - x0) * ir;
      end
      else
        y := x1;
      res.pon_e(k, y);
      Inc(k);
      pAcum_k := k * 1.0 / N;
    end
    else
    begin
      if jBuscando < Np then
      begin
        x0 := x1;
        pAcum_jm1 := pAcum_j;
        Inc(jBuscando);
        x1 := x.e(jBuscando);
        pAcum_j := fdpAcum.e(jBuscando);
        delta_pAcum_j := pAcum_j - pAcum_jm1;
      end
      else
      begin
        y := x1;
        res.pon_e(k, y);
        Inc(k);
        pAcum_k := k * 1.0 / N;
      end;
    end;
  end;

  fdpAcum.Free;
  Result := res;

end;


//PA 28-01-2008
//Aproximacin de la funcin Gamma (extensin de factorial para los complejos)
//El valor z debe ser > 0
//El error en esta aproximacin esta acotado por
//|e| < 2 * 10^-10
//http://www.rskey.org/gamma.htm
function LanczosGammaAprox(z: NReal): NReal;
const
  raizDeDosPi = 2.506628274631000502415765284811;
  ps: array [0..6] of NReal =
    (1.000000000190015, 76.18009172947146, -86.50532032941677,
    24.01409824083091, -1.231739572450155, 1.208650973866179E-3,
    -5.395239384953E-6);
var
  acum: NReal;
  n: integer;
begin
  //p0 + sum(n=1..6) (pn / z + n)
  acum := ps[0];
  for n := 1 to 6 do
    acum := acum + ps[n] / (z + n);
  Result := (raizDeDosPi / z) * acum * power((z + 5.5), z + 0.5) * exp(-(z + 5.5));
end;

// la fddpActiva es para apuntar el objeto cuando llamamos
// el mtodo de Romberg o dicot para el clculo de area_t o t_area
var
  fddpActiva: Tf_ddp;

function fd(x: NReal): NReal;
begin
  fd := fddpActiva.densidad(x);
end;

function g(t: NReal): NReal;
begin
  g := fddpActiva.area_t(t) - fddpActiva.g0;
end;

// Procedimientos generales de acceso a la madreUniforme
function random: NReal;
begin
  raise Exception.Create('Llamaron a fddp.random. Se debe crear una TMadreUniforme' +
    ' y utilizar el mtodo rnd');
  //    result:= madreUniforme.rnd;
  Result := 0;
end;

procedure FijarSemilla(NuevaSemilla: integer);
begin
  raise Exception.Create('Llamaron a fddp.FijarSemilla. Se debe crear una TMadreUniforme'
    + ' y utilizar el mtodo reiniciar');
  //  madreUniforme.Reiniciar(NuevaSemilla);
end;

function semillaAleatoria: integer;
begin
  raise Exception.Create(
    'Llamaron a fddp.semillaAleatoria. Se debe crear una TMadreUniforme'
    + ' y consultar el valor de semilla');
  Result := 0;
  //result:= -madreUniforme.semilla;
end;

procedure randomize;
begin
  raise Exception.Create('Llamaron a fddp.randomize. Se debe crear una TMadreUniforme' +
    ' y consultar el valor de semilla');
  //    system.randomize;
  //    madreUniforme.Reiniciar( RandSeed );
end;

{ mtodos de las Tf_ddp }

constructor Tf_ddp.Create(xt0, xarea_t0, xt1, xarea_t1: NReal; semilla: integer);
begin
  inherited Create;
  t0 := xt0;
  area_t0 := xarea_t0;
  t1 := xt1;
  area_t1 := xarea_t1;
  area_u_t := area_t0;
  u_t := t0;
  self.semilla_base := semilla;
  madreUnif := nil;
  flg_BorrarMadreUnif := False;
end;

constructor Tf_ddp.Create(xt0, xarea_t0, xt1, xarea_t1: NReal;
  madreUniforme: TMadreUniforme; semilla: integer);
begin
  inherited Create;
  t0 := xt0;
  area_t0 := xarea_t0;
  t1 := xt1;
  area_t1 := xarea_t1;
  area_u_t := area_t0;
  u_t := t0;
  if madreUniforme <> nil then
  begin
    madreUnif := madreUniforme;
    flg_BorrarMadreUnif := False;
  end
  else
  begin
    madreUnif := TMadreUniforme.Create(semilla);
    flg_BorrarMadreUnif := True;
  end;
end;

procedure Tf_ddp.Free;
begin
  if flg_BorrarMadreUnif then
    madreUnif.Free;
  inherited Free;
end;

function Tf_ddp.distribucion(x: NReal): NReal;
begin
  distribucion := area_t_rapida(x);
end;

function Tf_ddp.densidad(x: NReal): NReal;
begin
  densidad := 0;
end;

function Tf_ddp.area_t(t: NReal): NReal;
var
  sum: NReal;
  res: boolean;
begin
  fddpActiva := Self;
  Romberg(
    fd,
    t0, t, AsumaCero,
    sum,
    res);

  Result := sum + area_t0;
end;

function Tf_ddp.area_t_rapida(t: NReal): NReal;
{
  Calcula la integral en a partir del ltimo punto calculado
esto aumenta la velocidad cuando se calculan puntos sercanos. }
var
  sum: NReal;
  res: boolean;
  ts: NReal;
begin
  fddpActiva := Self;
  if t > u_t then
  begin
    ts := u_t + dtmax;
    while ts < t do
    begin
      Romberg(
        fd,
        u_t, ts, 1e-4,
        sum,
        res);
      area_u_t := sum + area_u_t;
      u_t := ts;
      ts := ts + dtmax;
    end;

    Romberg(
      fd,
      u_t, t, 1e-4,
      sum,
      res);
    area_u_t := sum + area_u_t;
    u_t := t;
    area_t_rapida := area_u_t;
  end
  else
  begin
    ts := u_t - dtmax;
    while ts > t do
    begin
      Romberg(
        fd,
        u_t, ts, 1e-4,
        sum,
        res);
      area_u_t := sum + area_u_t;
      u_t := ts;
      ts := ts - dtmax;
    end;

    Romberg(
      fd,
      u_t, t, 1e-4,
      sum,
      res);
    area_u_t := sum + area_u_t;
    u_t := t;
    area_t_rapida := area_u_t;
  end;
end;

procedure Tf_ddp.Reiniciar(nueva_semilla_base: integer);
begin
  if nueva_semilla_base > 0 then
    semilla_base := -nueva_semilla_base
  else
    semilla_base := nueva_semilla_base;
end;

function Tf_ddp.get_semilla_base: integer;
begin
  if semilla_base < 0 then
    Result := -semilla_base
  else
    Result := semilla_base;
end;




function Tf_ddp.t_area(area: NReal): NReal;
var
  t, gt: NReal;
  NoOfIts: word;
  converged: boolean;
begin
  if area <= area_t0 then
  begin
    Result := t0;
    exit;
  end;

  if area >= area_t1 then
  begin
    Result := t1;
    exit;
  end;

  fddpActiva := Self;
  g0 := area;

  Dicot(
    g,                    {funcion a anular}
    t0, t1, 1e-4,         {extremos y tolerancia}
    1000,                  {numero maximo de iteraciones}
    t, gt,                {raiz y f(raiz)}
    NoOfIts,              {numero de iteraciones realizadas}
    converged);            {validez del resultado}

  if not converged then
    raise Exception.Create(' Problema de convergencia , area: ' + FloatToStr(area));
  t_area := t;
end;

function Tf_ddp.rnd: NReal;
var
  u: NReal;
  tmps: integer;
begin
  u := self.madreUnif.rnd;
  Result := t_area(u);
end;


constructor Tf_ddp_GaussianaNormal.Create(madreUniforme: TMadreUniforme;
  semilla: integer);
begin
  inherited Create(-5, distribucion(-5), 5, distribucion(5), madreUniforme, semilla);
  gliset := 0;
  glgset := 0;
  dtmax := 1;
end;


function Tf_ddp_GaussianaNormal.densidad(x: NReal): NReal;
const
  usr2p = 0.3989422804;
var
  r: NReal;
begin
  r := x / 2;
  densidad := usr2p * exp(-(r * r));
end;

function Tf_ddp_GaussianaNormal.distribucion(x: NReal): NReal;
begin
  Result := DistribucionNormalCanonica(x);
end;


function Tf_ddp_GaussianaNormal.area_t(t: NReal): NReal;
begin
  Result := distribucion(t);
end;

function Tf_ddp_GaussianaNormal.area_t_rapida(t: NReal): NReal;
begin
  Result := distribucion(t);
end;

function Tf_ddp_GaussianaNormal.t_area(area: NReal): NReal;
begin
  Result := Inv_DistribucionNormalCanonica(area);
end;

function Tf_ddp_GaussianaNormal.rnd: NReal;
begin
  Result := Inv_DistribucionNormalCanonica(madreUnif.rnd);
end;

(*** saco esto porque no se reiniciaba bien con la semilla
VAR
  fac,r,v1,v2: NReal;
BEGIN
  IF  (gliset = 0)  THEN
  BEGIN
    REPEAT
      v1 := 2.0*madreUnif.rnd-1.0;
      v2 := 2.0*madreUnif.rnd-1.0;
      r := sqr(v1)+sqr(v2);
    UNTIL (r < 1.0);
    fac := sqrt(-2.0*ln(r)/r);
    glgset := v1*fac;
    result := v2*fac;
    gliset := 1
  END
  ELSE
  BEGIN
    result := glgset;
    gliset := 0
  END
END;
*)

procedure Tf_ddp_GaussianaNormal.Reiniciar(semilla: integer);
begin
  gliset := 0;
  inherited Reiniciar(semilla);
end;

{ Metedos de f_student }

constructor Tf_ddp_Student.Create(N: integer; madreUniforme: TMadreUniforme;
  semilla: integer);
var
  s: NReal;
begin
  inherited Create(0, 0.5, 100, 1, madreUniforme, semilla);
  dtmax := 1;
  nu := N - 1;
  y0 := 1;
  s := area_t_rapida(100);
  u_t := 0;
  area_u_t := 0.5;
  y0 := 1 / ((s - 0.5) * 2);
end;

function pot(x, y: NReal): NReal;
begin
  if x < 1e-12 then
    pot := 0
  else
    pot := exp(y * ln(x));
end;

function Tf_ddp_Student.densidad(x: NReal): NReal;
begin
  densidad := Y0 / pot(1 + x * x / nu, (nu + 1) / 2);
end;

{ metodos de f_ChiCuadrado = class(Tf_ddp) }

constructor Tf_ddp_ChiCuadrado.Create(N: integer; madreUniforme: TMadreUniforme;
  semilla: integer);
var
  s: NReal;
begin
  inherited Create(0, 0, 100, 1, madreUniforme, semilla);
  dtmax := 1;
  nu := N - 1;
  y0 := 1;

  s := area_t_rapida(300);
  u_t := 0;
  area_u_t := 0;
  y0 := 1 / s;
end;

function Tf_ddp_ChiCuadrado.densidad(x: NReal): NReal;
begin
  densidad := Y0 * pot(x, (nu - 2) / 2) * exp(-x / 2);
end;

{ mtodos de la funcin de probabilidad de Weibool }
constructor Tf_ddp_Weibull.Create(ValorMedio, Constante_k: NReal;
  madreUniforme: TMadreUniforme; semilla: integer);
begin
  inherited Create(0, 0, ValorMedio * 1000, 1, madreUniforme, semilla);
  setNuevosParams(ValorMedio, Constante_k);
end;


// estima a partir del vector de muestras los valores de
// k y lambda que mejor ajustan
constructor Tf_ddp_Weibull.Create_vmuestras(
  vdatos: TVectR;
  madreUniforme: TMadreUniforme; semilla: integer);

(***

u = 1 - exp( - (x/ lambda )^k )
Ln( x ) k - k L(lambda) = ln( ln( 1/(1-u) )

por mnimo cuadrados identifico
k y beta = (k L(lambda) )
luego
  lambda = exp( beta/k )
**)
var
  LLu, Lx: TVectR;
  k: integer;
  u: NReal;
  A, B: TMatR;
  res_inv: boolean;
  res_exp10: Integer;
  beta: NReal;
  z: NReal;


begin
  LLu:= TVectR.Create_Init( vdatos.n );
  Lx:= TVectR.Create_Init( vdatos.n );
  for k:= 1 to vdatos.n do
  begin
    u:= ( k - 0.5 ) /vdatos.n;
    z:=   1/(1- u);
    z:= Ln(z );
    z:= Ln( z );
    LLu.pon_e( k, z );
    Lx.pon_e( k, Ln( vdatos.e( k ) ) );
  end;
  A:= TMatR.Create_Init( 2, 2 );
  B:= TMatR.Create_INit( 2, 1 );
  a.pon_e(1,1, Lx.PEV( Lx )/vdatos.n);
  a.pon_e(1,2, - Lx.promedio);
  b.pon_e(1,1, Lx.PEV( LLu )/vdatos.n);
  a.pon_e(2,1, a.e(1,2) );
  a.pon_e(2,2, 1 );
  b.pon_e(2,1, -LLu.promedio );

  A.Escaler( B, res_inv, res_exp10);
  if not res_inv then
    raise Exception.Create('Imposible invertir sistema Tf_ddp_Weibull.Create_vmuetras en fddp.pas');

  inherited Create(0, 0, vdatos.promedio * 1000, 1, madreUniforme, semilla);

  self.k:= B.e(1,1);
  beta:= B.e(2,1);

  A.Free;
  B.Free;

  self.lambda:= exp( beta/self.k );
  self.unoSobreK:= 1/ self.k;
  self.kSobreLambda:= self.k/self.lambda;
end;

procedure Tf_ddp_Weibull.setNuevosParams(ValorMedio, Constante_k: NReal);
var
  gamaaprox: NReal;
begin
  k := Constante_k;
  unoSobreK := 1.0 / Constante_k;
  //  gamaaprox:= 0.2869/k + 0.688*exp(-0.1*ln(k));
  gamaaprox := LanczosGammaAprox(1 + unoSobreK);
  lambda:= ValorMedio / gamaaprox;
  kSobreLambda := k / lambda;
end;

function Tf_ddp_Weibull.densidad(x: NReal): NReal;
var
  x_ala_k, x_sobre_lambda: NReal;
begin
  if x <= AsumaCero then
    densidad := 0
  else
  begin
    // pdf( X ) = (k/lambda) * (x/lambda)^(k-1) * exp( - (x/lambda)^k )
    x_sobre_lambda:= x / lambda;
    x_ala_k := power( x_sobre_lambda, k);
    densidad := kSobrelambda * (x_ala_k / x_sobre_lambda) * exp(-x_ala_k);
  end;
end;

function Tf_ddp_Weibull.distribucion(x: NReal): NReal;
var
  x_ala_k, x_sobre_lambda: NReal;

begin
  if x <= 0 then
    distribucion := 0
  else
  begin
    // cdf( X ) = 1 - exp( -1 (x/lambda)^k )
    x_sobre_lambda := x / lambda;
    x_ala_k := power(x_sobre_lambda, k);
    distribucion := 1 - exp(-x_ala_k);
  end;
end;

function Tf_ddp_Weibull.area_t(t: NReal): NReal;
begin
  Result := distribucion(t);
end;

function Tf_ddp_Weibull.area_t_rapida(t: NReal): NReal;
begin
  Result := distribucion(t);
end;

function Tf_ddp_Weibull.t_area(area: NReal): NReal;
var
  x_ala_k: NReal;
begin
  if area <= AsumaCero then
    Result := 0
  else
  begin
    x_ala_k := -ln(1 - area);
    Result := power(x_ala_k, unoSobreK) * lambda;
  end;
end;




constructor Tf_ddp_VectDeMuestras.Create;  // usar uno de los dos siguientes
begin
  raise Exception.Create('Tf_dpp_VectDeMuestras.Create --- Usar la versin Clonando');
end;
// esta versin hace una copia del vector muestras.

constructor Tf_ddp_VectDeMuestras.Create_ClonandoMuestras(muestras: TVectR;
  madreUniforme: TMadreUniforme; semilla: integer);
begin
  inherited Create(0, 0, 1, 1, madreUniforme, semilla); // arreglamos t0  y t1 despus
  a := TVectR.Create_Init(muestras.n);
  a.Copy(muestras);
  a.Sort(True);
  Self.t0 := a.e(1);
  Self.t1 := a.e(a.n);
end;


constructor Tf_ddp_VectDeMuestras.Create_sinClonarMuestras(muestras: TVectR;
  madreUniforme: TMadreUniforme; semilla: integer);
begin
  inherited Create(0, 0, 1, 1, madreUniforme, semilla); // arreglamos t0  y t1 despus
  self.a := muestras;
  a.Sort(True);
  Self.t0 := a.e(1);
  Self.t1 := a.e(a.n);
end;

procedure Tf_ddp_VectDeMuestras.Free;
begin
  a.Free;
  inherited Free;
end;

function Tf_ddp_VectDeMuestras.k_inf_t(t: NReal): integer;
var
  k: integer;
  j: integer;
begin
  if (t < a.pv[1]) then
  begin
    Result := 0;
    exit;
  end;

{
  if (t >= a.pv[a.n]) then
  begin
    Result := a.n;
    exit;
  end;
 }

  k := ubicar_creciente_der(a.pv, 1, a.n, t);
  j:= k;
  while (j>2) and (a.pv[j-1] = t ) do dec( j );

  if j <> k then
  {$IFDEF RND_PROBABILIDAD_ACUMULADA}
   result:= trunc( self.rnd * ( k - j ) + j - 0.5)
   {$ELSE}
   result:= trunc( ( k + j ) / 2.0 - 0.5 )
   {$ENDIF}
  else
   result := k - 1;

end;

function Tf_ddp_VectDeMuestras.densidad(x: NReal): NReal;
var
  k1, k2: integer;
  t1, t2: NReal;
  dt: NReal;
  uk: NReal;
begin
  k1 := k_inf_t(x);
  if (k1 >= a.n) then
  begin
    Result := 0;
    exit;
  end;

  if (k1 < 1) then
  begin
    Result := 0;
    exit;
  end;

  t1 := a.e(k1);
  k2 := k1 + 1;
  t2 := a.e(k2);
  dt := t2 - t1;
  if dt > AsumaCero then
    uk := 1 / dt / (a.n - 1)
  else
    uk := 1e6; //ojo no est bien definido

  Result := uk;
end;

function Tf_ddp_VectDeMuestras.area_t(t: NReal): NReal;
var
  k1: integer;
  t1, t2: NReal;
  dt: NReal;
  uk: NReal;
begin
  k1 := k_inf_t(t);
  if (k1 >= a.n) then
  begin
    Result := 1;
    exit;
  end;

  if (k1 < 1) then
  begin
    t1 := a.pv[k1];
    Result := 0;
    exit;
  end;


  t1 := a.pv[k1];
  t2 := a.pv[k1+1];

  dt := t2 - t1;

{$IFDEF REVISAR_rch20110731}
  // as es como estaba
  if dt > AsumaCero then
    uk := (k1 + (t - t1) / dt - 1) / (a.n - 1)
  else
    uk := (k1 - 1) / (a.n - 1);
{$ELSE}
  // as es como me parece tendra que ser
  if dt > AsumaCero then
    uk := (k1 + (t - t1) / dt ) / a.n
  else
    uk := (k1 ) / a.n;
{$ENDIF}

  Result := uk;
end;

function Tf_ddp_VectDeMuestras.t_area(area: NReal): NReal;
var
  uk: NReal;
  k: integer;
begin
  uk := area * (a.n - 1);
  k := trunc(uk) + 1;

  (** cul ser?
  // as es como me parece tiene que ser
  uk := area * a.n;
  k := trunc(uk);
  **)

  if (k < 1) then
  begin
    Result := a.e(1);
    exit;
  end;

  if (k >= a.n) then
  begin
    Result := a.e(a.n);
    exit;
  end;

  uk := frac(uk);
  Result := a.pv[k] + uk * (a.pv[k + 1] - a.pv[k]);
end;

constructor Tf_ddp_VectFPAcum.Create(
  // el primer casillero es el de xt0, y el ltimo elde xt1
  // el primer casillero debiera tener valor 0 y el ltimo 1 para
  // que xt1, xt2 comprenda al rango posible de la variable
  FAcum: TVectR; xt0, xt1: NReal; madreUniforme: TMadreUniforme;
  semilla: integer);
begin
  inherited Create(xt0, FAcum.e(1), xt1, 1, madreUniforme, semilla);
  a := TVectR.Create_Init(FAcum.n);
  a.Copy(FAcum);

  dt := (t1 - t0) / (a.n - 1);
end;

procedure Tf_ddp_VectFPAcum.Free;
begin
  a.Free;
  inherited Free;
end;

(* retorna el k tal que a.e(k) <= u <= a.e(k+1)
  si retorna 0 significa que u < a.e(1)

  u debe estar entre [0, 1]
*)
function Tf_ddp_VectFPAcum.k_inf_u(u: NReal): integer;
var
  buscando: boolean;
  k: integer;
begin
  if u <= a.e(1) then
  begin
    Result := 1;
    exit;
  end;

  if u > a.e(a.n) then
  begin
    Result := a.n;
    exit;
  end;

  buscando := True;
  k := 1;
  while buscando and (k < a.n) do
  begin
    if a.e(k + 1) > u then
      buscando := False
    else
      Inc(k);
  end;

  Result := k;
end;

function Tf_ddp_VectFPAcum.densidad(x: NReal): NReal;
var
  uk: NReal;
  k: integer;
  F1, F2: NReal;
begin
  uk := (x - t0) / dt;
  k := trunc(uk) + 1;

  if k < 1 then
  begin
    Result := 0;
    exit;
  end;

  if k >= a.n then
  begin
    Result := 0;
    exit;
  end;

  F1 := a.pv[k];
  F2 := a.pv[k + 1];

  Result := (F2 - F1) / dt;
end;

function Tf_ddp_VectFPAcum.area_t(t: NReal): NReal;
var
  uk: NReal;
  k: integer;
  F1, F2: NReal;
begin
  uk := (t - t0) / dt;
  k := trunc(uk) + 1;

  if k < 1 then
  begin
    Result := 0;
    exit;
  end;

  if k >= a.n then
  begin
    Result := 1;
    exit;
  end;

  F1 := a.pv[k];
  F2 := a.pv[k + 1];

  uk := frac(uk);
  Result := F1 + (F2 - F1) * uk;
end;

function Tf_ddp_VectFPAcum.t_area(area: NReal): NReal;
var
  k: integer;
  F1, F2: NReal;
  dF: NReal;
begin
  k := k_inf_u(area);
  if k <= 1 then
  begin
    Result := t0;
    exit;
  end;

  if k >= a.n then
  begin
    Result := t1;
    exit;
  end;

  F1 := a.pv[k];
  F2 := a.pv[k + 1];

  dF := F2 - F1;
  if dF < AsumaCero then
  begin
    Result := t0 + (k - 1) * dt;
    exit;
  end;

  Result := t0 + (k - 1) * dt + (area - F1) / dF * dt;
end;

(********
Mtodos de Tf_ddp_F1mExpPol
*******)

constructor Tf_ddp_F1mExpPol.Create(xpol: TVectR; xt0, xt1: NReal;
  madreUniforme: TMadreUniforme; semilla: integer);
var
  k: integer;
begin
  inherited Create(xt0, 0, xt1, 1, madreUniforme, semilla);
  pol := xpol;
  mdpol := TVectR.Create_Init(pol.n - 1);
  for k := 1 to mdpol.n do
    mdpol.pon_e(k, -(k + 1) * pol.e(k + 1));
end;

procedure Tf_ddp_F1mExpPol.Free;
begin
  pol.Free;
  mdpol.Free;
  inherited Free;
end;

function Tf_ddp_F1mExpPol.densidad(x: NReal): NReal;
begin
  if (x < t0) or (x > t1) then
  begin
    Result := 0;
    exit;
  end;

  Result := pol.rpoly(x) * mdpol.rpoly(x);
end;

function Tf_ddp_F1mExpPol.area_t(t: NReal): NReal;
begin
  if (t < t0) then
  begin
    Result := 0;
    exit;
  end;

  if (t >= t1) then
  begin
    Result := 1;
    exit;
  end;

  Result := 1 - exp(pol.rpoly(t));
end;

// Mtodos de tf_dppMadreUniformeRand3
constructor TMadreUniforme.Create(semilla: integer);
{$IFDEF NO_USAR_RND_COMPILADOR}
var
  k: integer;
{$ENDIF}
begin
  inherited Create(0, 0, 1, 1, semilla);
  {$IFDEF NO_USAR_RND_COMPILADOR}
  glinext := 0;
  glinextp := 0;
  for k := 1 to 55 do
    glma[k] := 0;
  {$ENDIF}
  Reiniciar(semilla);
end;

procedure TMadreUniforme.Free;
begin
  inherited Free;
end;

function TMadreUniforme.densidad(x: NReal): NReal;
begin
  if x < 0 then
    Result := 0
  else if x >= 1 then
    Result := 0
  else
    Result := 1;
end;

function TMadreUniforme.distribucion(x: NReal): NReal;
begin
  if x < 0 then
    Result := 0
  else if x >= 1 then
    Result := 1
  else
    Result := x;
end;

function TMadreUniforme.area_t(t: NReal): NReal;
begin
  Result := distribucion(t);
end;

function TMadreUniforme.area_t_rapida(t: NReal): NReal;
begin
  Result := distribucion(t);
end;

function TMadreUniforme.t_area(area: NReal): NReal;
begin
  if area < 0 then
    Result := 0
  else if area >= 1 then
    Result := 1
  else
    Result := area;
end;

//La funcin da overflow si se reinicia la semilla aunque despus
//funciona correctamente, probablemente alguna de las cuentas que hace al entrar
//por IF (semilla < 0) THEN ... debiera ser cclica.
{$OverflowChecks Off}
function TMadreUniforme.rnd: NReal;
{$IFDEF NO_USAR_RND_COMPILADOR}

{$IFNDEF NO_OPTIM_UNIF_PRECALC}
const
  //iis[i] = (21 * i) mod 55
  iis: array [1..54] of integer = (21, 42, 8, 29, 50, 16, 37, 3, 24, 45, 11, 32,
    53, 19, 40, 6, 27, 48, 14, 35, 1, 22, 43, 9,
    30, 51, 17, 38, 4, 25, 46, 12, 33, 54, 20,
    41, 7, 28, 49, 15, 36, 2, 23, 44, 10, 31, 52,
    18, 39, 5, 26, 47, 13, 34);

  //iaux[i] = 1+((i+30) MOD 55)
  iaux: array [1..55] of integer = (32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42,
    43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53,
    54, 55, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
    12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
    23, 24, 25, 26, 27, 28, 29, 30, 31);

{$ENDIF}

{$IFDEF RAND3_REAL}// si la mquine tiene enteros de menos de 4 bytes
const
  mbig = 4.0e6;
  mseed = 1618033.0;
  mz = 0.0;
  fac = 2.5e-7; (* 1/mbig *)
var
  i, ii, k: integer;
  mj, mk: real;
{$ELSE}
const
  mbig = 1000000000;
  mseed = 161803398;
  mz = 0;
  fac = 1.0e-9;
var
  i, ii, k, mj, mk: integer;
{$ENDIF}
{$ENDIF}
begin

{$IFDEF NO_USAR_RND_COMPILADOR}
  if (semilla_base < 0) then
  begin
    mj := mseed + semilla_base;
{$IFDEF RAND3_REAL}// si la mquine tiene enteros de menos de 4 bytes
    (* The following IF block is mj := mj MOD mbig; for real variables. *)
    if mj >= 0.0 then
      mj := mj - mbig * trunc(mj / mbig)
    else
      mj := mbig - abs(mj) + mbig * trunc(abs(mj) / mbig);
{$ELSE}
    while mj < 0 do
      mj := mj + mbig;
    mj := mj mod mbig;
{$ENDIF}
    glma[55] := mj;
    mk := 1;
    for i := 1 to 54 do
    begin
{$IFDEF NO_OPTIM_UNIF_PRECALC}
      ii := 21 * i mod 55;
{$ELSE}
      ii := iis[i];
{$ENDIF}
      glma[ii] := mk;
      mk := mj - mk;
      if (mk < mz) then
        mk := mk + mbig;
      mj := glma[ii];
    end;

    for k := 1 to 4 do
    begin
      for i := 1 to 55 do
      begin
       {$IFDEF NO_OPTIM_UNIF_PRECALC}
        glma[i] := glma[i] - glma[1 + ((i + 30) mod 55)];
       {$ELSE}
        glma[i] := glma[i] - glma[iaux[i]];
       {$ENDIF}
        if (glma[i] < mz) then
          glma[i] := glma[i] + mbig;
      end;
    end;
    glinext := 0;
    glinextp := 31;
    semilla_base := -semilla_base;
  end;

  glinext := glinext + 1;
  if (glinext = 56) then
    glinext := 1;
  glinextp := glinextp + 1;
  if (glinextp = 56) then
    glinextp := 1;
  mj := glma[glinext] - glma[glinextp];

  if (mj < mz) then
    mj := mj + mbig;

  glma[glinext] := mj;
  Result := mj * fac;

{$ELSE}
  Result := system.random;
{$ENDIF}
end;

function TMadreUniforme.randomInt: integer;
const
  largoRango: int64 = 4294967295;
var
  randomCeroUno: NReal;
begin
  randomCeroUno := rnd;
  Result := (-MaxInt - 1) + trunc(randomCeroUno * largoRango + 0.5);
end;

function TMadreUniforme.randomIntRange(const minRango, maxRango: integer): integer;
var
  randomCeroUno: NReal;
  largoRango: int64;
begin
  randomCeroUno := rnd;
  largoRango := maxRango - minRango;
  Result := minRango + trunc(randomCeroUno * largoRango + 0.5);
end;

procedure TMadreUniforme.Reiniciar(semilla: integer);
begin
{$IFDEF NO_USAR_RND_COMPILADOR}
  if semilla > 0 then
    Self.semilla_base := -semilla
  else
    Self.semilla_base := semilla;
{$ELSE}
  self.semilla_base := semilla;
  self.semilla := semilla;
{$ENDIF}
end;

procedure TMadreUniforme.setEstado(const estado: TEstadoMadreUniforme);
{$IFDEF NO_USAR_RND_COMPILADOR}
var
  i: integer;
{$ENDIF}
begin
  semilla_base := estado.semilla_base;
 {$IFDEF NO_USAR_RND_COMPILADOR}
  for i := low(glma) to High(glma) do
     glma[i] := estado.glma[i];
  glinext := estado.glinext;
  glinextp := estado.glinextp;
{$ELSE}
  Reiniciar(estado.semilla);
{$ENDIF}
end;

procedure TMadreUniforme.setEstado(const otraFuente: TMadreUniforme);
{$IFDEF NO_USAR_RND_COMPILADOR}
var
  i: integer;
{$ENDIF}
begin
  semilla_base := otraFuente.semilla_base;
 {$IFDEF NO_USAR_RND_COMPILADOR}
  for i := low(glma) to High(glma) do
    glma[i] := otraFuente.glma[i];
  glinext := otraFuente.glinext;
  glinextp := otraFuente.glinextp;
{$ELSE}
  Reiniciar(otraFuente.semilla);
{$ENDIF}
end;

procedure TMadreUniforme.getEstado(var estado: TEstadoMadreUniforme);
{$IFDEF NO_USAR_RND_COMPILADOR}
var
  i: integer;
{$ENDIF}
begin
  estado.semilla_base := Self.semilla_base;
  {$IFDEF NO_USAR_RND_COMPILADOR}
  for i := low( glma)  to High(glma) do
    estado.glma[i] := glma[i];
  estado.glinext := glinext;
  estado.glinextp := glinextp;
{$ELSE}
  estado.semilla_base := semilla_base;
  estado.semilla := semilla;
{$ENDIF}
end;

// Mtodos de tf_dppUniformeRand3
constructor Tf_ddpUniformeRand3.Create(madreUniforme: TMadreUniforme;
  semilla: integer);
begin
  inherited Create(0, 0, 1, 1, madreUniforme, semilla);
end;

function Tf_ddpUniformeRand3.densidad(x: NReal): NReal;
begin
  if x < 0 then
    Result := 0
  else if x >= 1 then
    Result := 0
  else
    Result := 1;
end;

function Tf_ddpUniformeRand3.distribucion(x: NReal): NReal;
begin
  if x < 0 then
    Result := 0
  else if x >= 1 then
    Result := 1
  else
    Result := x;
end;

function Tf_ddpUniformeRand3.area_t(t: NReal): NReal;
begin
  Result := distribucion(t);
end;

function Tf_ddpUniformeRand3.area_t_rapida(t: NReal): NReal;
begin
  Result := distribucion(t);
end;

function Tf_ddpUniformeRand3.t_area(area: NReal): NReal;
begin
  if area < 0 then
    Result := 0
  else if area >= 1 then
    Result := 1
  else
    Result := area;
end;

function Tf_ddpUniformeRand3.rnd: NReal;
begin
  Result := madreUnif.rnd;
end;

end.
